<?php

/**
 * @file
 * Administration pages for the PIFT module.
 */

/**
 * Form constructor for the PIFT settings form.
 *
 * @see pift_admin_settings_form_validate()
 * @see pift_admin_settings_form_submit()
 *
 * @ingroup forms
 */
function pift_admin_settings_form($form, &$form_state) {
  $form = array();

  // Issue states come from the field_issue_status field.
  $field = field_info_field('field_issue_status');

  // General settings container.
  $form['general'] = array(
    '#type' => 'fieldset',
    '#title' => t('General'),
  );
  // Control when tests are sent to the PIFR server.
  $form['general']['pift_frequency'] = array(
    '#type' => 'select',
    '#title' => t('Send frequency'),
    '#description' => t('Frequency at which test requests will be sent and test results retrieved.'),
    '#options' => array(
      -1 => t('Disabled'),
      0 => t('Cron run'),
      01 * 60 * 60 => t('Hourly'),
      24 * 60 * 60 => t('Daily'),
    ),
    '#default_value' => PIFT_FREQUENCY,
    '#required' => TRUE,
  );
  // Unique identifier used to identify this project client to the PIFR server.
  $form['general']['pift_key'] = array(
    '#type' => 'textfield',
    '#title' => t('Client key'),
    '#description' => t('The key assigned to this client.'),
    '#default_value' => PIFT_KEY,
    '#required' => TRUE,
  );
  // URI for the PIFR server this PIFT client should send test request to.
  $form['general']['pift_server'] = array(
    '#type' => 'textfield',
    '#title' => t('Server URL'),
    '#description' => t('The URL of the test server, include ending forward slash.'),
    '#default_value' => PIFT_SERVER,
    '#required' => TRUE,
  );
  // User interface settings container.
  $form['interface'] = array(
    '#type' => 'fieldset',
    '#title' => t('User interface'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  // Text to append onto the 'attach file' field description.
  $form['interface']['pift_description'] = array(
    '#type' => 'textarea',
    '#title' => t('File description'),
    '#description' => t('Additional text to be displayed on the project issue attachment fieldset.'),
    '#default_value' => PIFT_DESCRIPTION,
  );
  // Controls what to do with issue status when a test fails.
  $criteria = array(
    t('The file has failed testing.'),
    t('The issue the file is attached to is still passing the file testing criteria.'),
    t('The file is the last testable file posted to the issue.'),
  );
  $form['interface']['pift_followup_fail'] = array(
    '#type' => 'select',
    '#title' => t('Followup issue status - fail'),
    '#description' => t('The status to set an issue to when a test result meets all of the following criteria.' . theme('item_list', array('items' => $criteria))),
    '#default_value' => PIFT_FOLLOWUP_FAIL,
    '#options' => list_allowed_values($field),
    '#required' => TRUE,
  );
  // Controls what to do with issue status when a retest request is received.
  $form['interface']['pift_followup_retest'] = array(
    '#type' => 'select',
    '#title' => t('Followup issue status - retest'),
    '#description' => t('The status to set an issue to when someone requests a re-test and the issue currently does not fit the test criteria.'),
    '#default_value' => PIFT_FOLLOWUP_RETEST,
    '#options' => list_allowed_values($field),
    '#required' => TRUE,
  );

  // Removing for the D7 port, as the plan is to move the 'Automated Testing'
  // tab interface over to the Conduit project
  //$project_types = array();
  //array();
  //foreach (project_project_node_types() as $key => $project_type) {
  //  $project_types[$project_type] = $project_type;
  //}
  //$form['results'] = array(
  //  '#type' => 'fieldset',
  //  '#title' => t('Results Display'),
  //  '#weight' => 0,
  //  '#collapsible' => TRUE,
  //  '#collapsed' => TRUE,
  //);
  //$form['results']['pift_results_display'] = array(
  //  '#type' => 'checkboxes',
  //  '#title' => t('Project types'),
  //  '#description' => t('Project types for which the "Testing Status" tab should be displayed on the project page.'),
  //  '#options' => $project_types,
  //  '#default_value' => variable_get('pift_results_display', array()),
  //);

  // File Test Criteria settings container
  $form['criteria'] = array(
    '#type' => 'fieldset',
    '#title' => t('Criteria'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  // Defines the regular expression file mask criteria.
  $form['criteria']['pift_regex'] = array(
    '#type' => 'textfield',
    '#title' => t('File regex'),
    '#description' => t('Full regex pattern that a filename must match for the file to be tested.'),
    '#default_value' => PIFT_REGEX,
    '#required' => TRUE,
  );
  // Defines project_issue issue status criteria.
  $form['criteria']['pift_status'] = array(
    '#type' => 'select',
    '#multiple' => TRUE,
    '#title' => t('Issue statuses'),
    '#description' => t('List of issue statuses that an issue must belong to one of in order to test files.'),
    '#default_value' => variable_get('pift_status', array()),
    '#options' => list_allowed_values($field),
    '#required' => TRUE,
  );
  // Defines the core compatibility criteria.
  $form['criteria']['pift_core'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Core branches'),
    '#description' => t('Drupal core branches that are to be tested. Issues pertaining to them will have files tested.'),
    '#options' => pift_compatibility_list(),
    '#default_value' => variable_get('pift_core', array()),
  );
  // Release must be compatible with Drupal 6 or later in order to be tested.
  foreach ($form['criteria']['pift_core']['#options'] as $key => $option) {
    if ($option[0] < 6) {
      unset($form['criteria']['pift_core']['#options'][$key]);
    }
  }
  // Advanced settings container.
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  // Defines the 'base' project used in testing.
  $project = node_load(PIFT_PID);
  $form['advanced']['pift_pid'] = array(
    '#type' => 'textfield',
    '#title' => t('Core'),
    '#description' => t('Project that contains the core code required by modules to function.'),
    '#autocomplete_path' => 'project/autocomplete/issue/project',
    '#default_value' => ($project ? $project->title : ''),
    '#required' => TRUE,
  );
  // Defines file result expiry (i.e. retest interval). Currently disabled.
  $form['advanced']['pift_retest'] = array(
    '#type' => 'select',
    '#title' => t('Re-test interval'),
    '#description' => t('Number of days to wait before re-testing a previously passed file.'),
    '#options' => array(
      -1,
      24 * 60 * 60,
      48 * 60 * 60,
      72 * 60 * 60,
      96 * 60 * 60,
    ),
    '#default_value' => PIFT_RETEST,
    '#required' => TRUE,
  );
  $form['advanced']['pift_retest']['#options'] = drupal_map_assoc($form['advanced']['pift_retest']['#options'], 'format_interval');
  $form['advanced']['pift_retest']['#options'][-1] = t('Disabled');
  // Defines the base URL location for the git repositories.
  $form['advanced']['pift_git_base_url'] = array(
    '#type' => 'textfield',
    '#title' => t('PIFT Git Base URL'),
    '#description' => t('Base URL for Git repositories, like git://git.drupal.org/project/'),
    '#default_value' => variable_get('pift_git_base_url', 'git://git.drupal.org/project/'),
    '#required' => TRUE,
  );

  $form['#submit'] = array('pift_admin_settings_form_submit');
  return system_settings_form($form);
}

/**
 * Form validation handler for pift_admin_settings_form().
 *
 * @see pift_admin_settings_form_submit()
 */
function pift_admin_settings_form_validate($form, &$form_state) {
  // Validate pift key length
  if (drupal_strlen($form_state['values']['pift_key']) != 32) {
    form_set_error('client_key', t('Key must be 32 characters long.'));
  }
  // Validate pift server URL
  if (!$form_state['values']['pift_server'] || !@parse_url($form_state['values']['pift_server'])) {
    form_set_error('pift_server', t('Server URL must be a complete URL, schema included.'));
  }
  else if (drupal_substr($form_state['values']['pift_server'],
                          drupal_strlen($form_state['values']['pift_server']) - 1) != '/') {
    form_set_error('pift_server', t('Server URL must end with a slash.'));
  }
}

/**
 * Form submission handler for pift_admin_settings_form().
 *
 * @see pift_admin_settings_form_validate()
 */
function pift_admin_settings_form_submit($form, &$form_state) {
  // Determine the project id, based on the node title.
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', project_project_node_types(), 'IN')
    ->propertyCondition('status', 1)
    ->propertyCondition('title', $form_state['values']['pift_pid'])
    ->range(0,1);
  $result = $query->execute();

  // Add the project node ID to the submitted form.
  if (isset($result['node'])) {
    $project = reset($result['node']);
    $form_state['values']['pift_pid'] = $project->nid;
  }
}

