<?php

/**
 * @file
 * Cron functions for the PIFT module.
 */

/**
 * Retest tests older than the retest interval.
 *
 * Mark files for re-testing if they still meet all the criteria and the
 * re-test interval has passed.
 *
 * TODO:  Update the retest query, since the upload table doesn't exist in
 * Drupal 7 anymore.
 * TODO:  Update for the id => nid data move within pift_data
 */
/*
function pift_cron_retest() {
  if (PIFT_RETEST == -1) {
    return;
  }

  $api_versions = pift_core_api_versions();
  $sids = variable_get('pift_status', array());

  // Only attempt query if both values are not empty.
  if ($api_versions && $sids) {
    $retest_time = REQUEST_TIME - PIFT_RETEST;

    // Loop over 'ON' clause to remove an 'OR' which should be more efficient.
    foreach (array('u.nid = pi.nid', 'cu.nid = pi.nid') as $clause) {

      db_query("UPDATE {pift_data}
                SET status = :status
                WHERE type = :type
                AND id IN (
                  SELECT f.fid
                  FROM {files} f
                  LEFT JOIN {upload} u
                    ON f.fid = u.fid
                  LEFT JOIN {comment_upload} cu
                    ON f.fid = cu.fid

                  JOIN {project_issues} pi
                    ON " . $clause . "
                  JOIN {pift_project} p
                    ON pi.pid = p.pid
                  JOIN {project_release_nodes} r
                    ON pi.rid = r.nid

                  JOIN {node} n
                    ON r.nid = n.nid
                  JOIN {taxonomy_term_node} t
                    ON (n.vid = t.vid AND t.tid IN (:api_versions))

                  WHERE pi.sid IN (:sids)
                )
                AND status > :status2
                AND last_tested < :last_tested",
                array(
                  ':status' => PIFT_STATUS_QUEUE,
                  ':type' => PIFT_TYPE_FILE,
                  ':api_versions' => $api_versions,
                  ':sids' => $sids,
                  ':status2' => PIFT_STATUS_SENT,
                  ':last_tested' => $retest_time,
                ));
    }
  }
}
*/

/**
 * Create test batch and send it to the PIFR testing server.
 *
 * @return array
 *   Structure of returned array:
 *     - Success: $response = array('branches' => array(), 'files' => array());
 *       - Items in branches sub-arrays are
 *         array(nid/rid of project release node => test id on qa)
 *       - Items in files sub-arrays are
 *         array(fid of file => test id on qa)
 *     - Failure: $response = array('response' => 3, 'errors' => array()):
 *       - Items in errors array are strings describing the failure.
 *     - Complete failure: $response === FALSE
 */
function pift_cron_queue_batch() {
  // Instantiate batch counter
  $i = 1;
  // Build up the batch array. If there are file or branch tests to be sent,
  // then $batch['projects'] will be defined and non-empty.
  while (($batch = pift_cron_queue_batch_build()) && $batch['projects']) {
    // Log the send attempt to the database logs
    watchdog('pift', 'Sending batch to server %server: <pre>%batch</pre>', array('%server' => PIFT_SERVER, '%batch' => print_r($batch, TRUE)));

    // Send the batch request to the PIFR server and collect the response.
    $response = xmlrpc(PIFT_SERVER . 'xmlrpc.php', array('pifr.queue' => array(PIFT_KEY, $batch)));
    // If no response from the server, an error occured.
    if ($response === FALSE) {
      watchdog('pift', 'Failed to send test queue requests to server.', array(), WATCHDOG_ERROR);
      break;
    }
    // If $response['response'] is set, then it contains an error message.
    elseif (isset($response['response'])) {
      watchdog('pift', 'Test queue request did not succeed: @message. full response=<pre>@response</pre>',
        array('@message' => pift_cron_response_code($response['response']), '%response' => print_r($response, TRUE)), WATCHDOG_ERROR);
      break;
    }
    // $response['branches'] and $response['files'] contain the list of test
    // results returned by the PIFR server.
    elseif (isset($response['branches']) && isset($response['files'])) {
      // Store branch test IDs using the client_identifier (branch NID).
      foreach ($batch['branches'] as $branch) {
        if (isset($response['branches'][$branch['client_identifier']])) {
          pift_test_sent($response['branches'][$branch['client_identifier']], PIFT_TYPE_BRANCH, $branch['client_identifier']);
        }
      }
      // Store file test IDs using the client_identifier (file ID in this case).
      foreach ($batch['files'] as $file) {
        if (isset($response['files'][$file['client_identifier']])) {
          pift_test_sent($response['files'][$file['client_identifier']], PIFT_TYPE_FILE, $file['client_identifier']);
        }
      }
    }
    // Some other non-defined error has occured.
    else {
      watchdog('pift', 'Invalid response to test queue request.', $response, WATCHDOG_ERROR);
      break;
    }

    // Once the maximum number of batches has been sent break out of the loop
    // and save the rest for the next cron run.
    if ($i++ == PIFT_XMLRPC_MAX_BATCHES) {
      break;
    }
  }
}

/**
 * Build a batch of tests to queue.
 *
 * @return array
 *   Complete batch array as defined by pifr.queue().
 *   Structure of $batch:
 *     array('branches' => array(), 'files' => array(), 'projects' => array()).
 *
 *   - Structure of 'branches' sub-arrays:
 *     - 'project_identifier' => nid of project node associated with branch.
 *     - 'client_identifier' => nid of associated project release node
 *     - 'vcs_identifier' => git branch to be tested (7.x, 7.x-1.x, etc).
 *     - 'dependency' => comma-delimited list of project dependencies
 *     - 'plugin_argument' => array of extra information delivered with test
 *         plugin. For example, array('drupal.core.version' => 7).
 *     - 'test' => TRUE if the branch should be tested on its own (i.e. when a
 *         commit has happened)
 *     - 'link' => href of the project release node.
 *
 *   - Structure of the 'files' sub-arrays:
 *     - 'client_identifier' => fid of the patchfile in the files table.
 *     - 'file_url' => the relative URL of the file in the files directory.
 *     - 'branch_identifier' => the nid of the release node for the branch
 *         to test.
 *     - 'link' => full-qualified URL to the issue comment where the patch is
 *         uploaded.
 *
 *   - Structure of the 'projects' sub-arrays:
 *     - 'client_identifier' => nid of the project node.
 *     - 'name' => title of the project node.
 *     - 'repository_type' => "git".
 *     - 'link' => fully-qualified link to the project node.
 */
function pift_cron_queue_batch_build() {
  // Initialize basic batch structure.
  $batch = array(
    'branches' => array(),
    'files' => array(),
    'projects' => array(),
  );

  // Provide arrays to keep track of the required branches and projects.
  $branches = array();
  $projects = array();

  // Load files that need to be reviewed.
  pift_cron_queue_batch_build_files($batch, $branches);

  // Load branches that need to be reviewed.
  pift_cron_queue_batch_build_branches($branches);

  // $branches is just an array of array(rid => TRUE) for each
  // release branch which is to be tested.

  // Process the required branches and add related projects.
  pift_cron_queue_batch_build_branches_process($batch, $branches, $projects);

  // Generate project information for each required project.
  pift_cron_queue_batch_build_projects($batch, $projects);

  return $batch;
}

/**
 * Load files that need to be reviewed and add them to the batch.
 *
 * @param array $batch
 *   An array of existing batch information.
 * @param array $branches
 *   An array of branch information associated with this batch.
 */
function pift_cron_queue_batch_build_files(array &$batch, array &$branches) {
  // Retrieve all file tests which are marked as needs testing.
  $tests = pift_test_get_outstanding_tests('file');

  if (!empty($tests)) {
    $fids = pift_extract_key($tests, 'id');
    $issue_nids = pift_extract_key($tests, 'nid');
    // Load the issue nodes for each file marked as needs testing.
    $issues = entity_load('node', $issue_nids);

    // Foreach issue, datafill the file test batch array placeholder item
    foreach ($issues as $issue) {

      $pid = $issue->field_project[LANGUAGE_NONE][0]['target_id'];
      $version = $issue->field_issue_version[LANGUAGE_NONE][0]['value'];
      $rid = project_release_exists($pid, $version);

      $item['branch_identifier'] = $rid;

      // Store the branch for the file test in the 'branches' array as needed.
      // We mark the value 'FALSE' to indicate that we do not need to test this
      // branch, unless a commit is found on that branch later in processing.
      $branches[$rid] = FALSE;

      // Add any additional branches which this branch is dependent on to the
      // 'branches' array.
      $branch_dependencies = project_dependency_get_dependencies($rid, PROJECT_DEPENDENCY_DEPENDENCY_REQUIRED);
      foreach ($branch_dependencies as $dep) {
        $branches[$dep['release_nid']] = FALSE;
      }

      // Add each relevant file on this issue to the batch
      foreach ($issue->field_issue_files[LANGUAGE_NONE] as $file) {
        if (in_array($file['fid'], $fids)) {
          // Finish datafilling the placeholder item.
          $item['client_identifier'] = $file['fid'];
          $item['file_url'] = file_create_url($file['uri']);
          $item['link'] = url('node/' . $issue->nid, array('absolute' => TRUE));
          // Add the placeholder item to the batch.
          $batch['files'][] = $item;
        }
      }
    }
  }
}

/**
 * Load branches that need to be reviewed and add them to the batch.
 *
 * @param array $branches
 *   An array of branch information associated with this batch.
 */
function pift_cron_queue_batch_build_branches(array &$branches) {
  // Load branches that are marked as needs testing or have never been tested.
  $tests = pift_test_get_outstanding_tests('branch');
  if (!empty($tests)) {
    $rids = pift_extract_key($tests, 'nid');
    foreach ($rids as $rid) {
      $branches[$rid] = TRUE;
    }
  }
}

/**
 * Process the array of associated branch information and add related projects.
 *
 * @param array $batch
 *   An array of existing batch information.
 * @param array $branches
 *   An array of branch information associated with this batch.
 * @param array $projects
 *   An array of project IDs associated with this batch.
 * @param integer $depth
 *   (Internal) Counter tracking the number of times the function has been
 *   recursively executed.
 */
function pift_cron_queue_batch_build_branches_process(array &$batch, array &$branches, array &$projects, $depth = 0) {

  $api_versions = pift_core_api_versions();

  // Include branch data for all dependency branches and those that require review.
  $branches_implied = array();
  foreach ($branches as $rid => $test) {
    // Load branch release node.
    if (!($branch = node_load($rid))) {
      watchdog('pift', 'Invalid release ID [%rid] for test id [%test_id].', array('%rid' => $rid, '%test_id' => $test['test_id']), WATCHDOG_ERROR);
      continue;
    }

    // Determine the parent project nid for this branch
    $pid = $branch->field_release_project[LANGUAGE_NONE][0]['target_id'];

    // Generate branch information.
    $item = array(
      'project_identifier' => $pid,
      'client_identifier' => $branch->nid,
      'vcs_identifier' => $branch->field_release_vcs_label[LANGUAGE_NONE][0]['value'],
      'dependency' => '',
      'plugin_argument' => array(),
      'test' => $test,
      'link' => url('node/' . $branch->nid, array('absolute' => TRUE)),
    );

    // Determine the Drupal core API version.
    $api = array();
    $term = taxonomy_term_load(project_release_get_release_api_tid($branch));
    if (in_array($term->tid, $api_versions)) {
      $api['version'] = array_shift(explode('.', $term->name, 2));
      $api['tid'] = $term->tid;
    }
    // If the API version not found then ignore this branch.
    else {
      watchdog('pift', 'Project release node [@nid] does not have a Drupal core API taxonomy term.', array('@nid' => $branch->nid), WATCHDOG_ERROR);
    }

    // If the project is Drupal core then add the plugin argument, otherwise
    // determine the compatible core branch and add it as a dependency.
    if (PIFT_PID == $item['project_identifier']) {
      $item['plugin_argument']['drupal.core.version'] = $api['version'];
    }
    else {
      // Load the Drupal core API release (branch) compatible with this branch.
      $item['dependency'][] = pift_core_api_release($api['tid']);

      // Cycle through dependencies and add the related branches to the implied
      // branch list if they are not already being processed.
      // We do not have to get new dependencies when we're looking at implied
      // branches.
      if ($depth == 0) {
        $dependencies = project_dependency_get_external_release_dependencies($branch->nid);
        foreach ($dependencies as $release_nid => $dependency) {
          $item['dependency'][] = $release_nid;
        }

        // Check for branches as dependencies that are not in the list of
        // processed branches.
        foreach ($item['dependency'] as $rid) {
          // Add branch to implied list for post-processing.
          if (!isset($branches[$rid])) {
            $branches_implied[$rid] = FALSE;
          }
        }
      }
      // Ensure that not depending on self.
      unset($item['dependency'][$branch->nid]);

      // Flatten array of dependencies and fill in test.directory.review.
      $item['dependency'] = implode(',', $item['dependency']);

      // Set the review directory to the project URI.
      $project = node_load($pid);
      $project_name = $project->field_project_machine_name[LANGUAGE_NONE][0]['value'];
      $item['plugin_argument']['test.directory.review'][] = 'sites/default/modules/' . $project_name;
    }

    // Add item information to the batch.
    $batch['branches'][] = $item;

    // Add branch's project to list to be loaded.
    $projects[$pid] = $pid;
  }

  // Post-process any implied branches.
  if ($branches_implied && $depth == 0) {
    pift_cron_queue_batch_build_branches_process($batch, $branches_implied, $projects, $depth + 1);
  }
}

/**
 * Generate project information for each project and add it to the batch.
 *
 * @param array $batch
 *   An array of existing batch information.
 * @param array $projects
 *   An array of project IDs associated with this batch.
 */
function pift_cron_queue_batch_build_projects(array &$batch, array $projects) {
  // Cycle through each required project.
  foreach ($projects as $pid) {
    // Load project node.
    $project = node_load($pid);

    // Generate project information.
    $item = array(
      'client_identifier' => $project->nid,
      'name' => $project->title,
      'repository_type' => 'git',
      'repository_url' => variable_get('pift_git_base_url', 'git://git.drupal.org/project/') . $project->field_project_machine_name[LANGUAGE_NONE][0]['value'] . '.git',
      'link' => url('node/' . $project->nid, array('absolute' => TRUE)),
    );

    $batch['projects'][] = $item;
  }
}

/**
 * Get the release NIDs that correspond to the specified project and tags.
 *
 * @param integer $pid
 *   The project node ID for the project being queried.
 * @param array $tags
 *   An array of tags being queried against.
 *
 * @return array
 *   An array of Release node IDs that match the specified project and tags.
 */
function pift_cron_get_release($pid, $tags) {
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', 'project_release')
    ->propertyCondition('status', 1)
    ->fieldCondition('field_release_project', 'target_id', $pid, '=')
    ->fieldCondition('field_release_vcs_label', 'value', $tags, 'IN');
  $result = $query->execute();
  $rids = array();
  if (isset($result['node'])) {
    $rids = array_keys($result['node']);
  }
  return $rids;
}

/**
 * Retrieve test results which have completed since the last cron run.
 */
function pift_cron_retrieve_results() {
  $next = PIFT_LAST_RETRIEVE;
  while ($next) {
    $response = xmlrpc(PIFT_SERVER . 'xmlrpc.php', array('pifr.retrieve' => array(PIFT_KEY, (int) $next)));
    $next = FALSE;

    if ($response === FALSE) {
      watchdog('pift', 'Failed to retrieve test results from to server: @message',
                       array('@message' => xmlrpc_error_msg()), WATCHDOG_ERROR);
    }
    elseif (isset($response['response'])) {
      if ($response['response'] != PIFR_RESPONSE_ACCEPTED) {
        watchdog('pift', 'Retrieval of test results did not succeed: @message.',
               array('@message' => pift_cron_response_code($response['response'])), WATCHDOG_ERROR);
      }
    }
    elseif (isset($response['results'])) {
      foreach ($response['results'] as $result) {
        // Update the test record with the results.
        pift_test_update_result($result);
        // Check if an auto followup comment is applicable and create one if
        // required.
        pift_cron_check_auto_followup($result);
      }

      // If there are more results then set $next to the last timestamp to be
      // used in next query.
      $next = $response['next'] ? $response['last'] : FALSE;

      // Always store last timestamp for next cron run.
      variable_set('pift_last_retrieve', $response['last']);
    }
    else {
      watchdog('pift', 'Invalid response to retrieval of test results.', array(), WATCHDOG_ERROR);
    }
  }
}

/**
 * Check if an auto followup comment is applicable, and if so, create it.
 *
 * @param array $result
 *   An array of test result information from the XML-RPC response.
 */
function pift_cron_check_auto_followup(array $result) {
  if (!$result['pass']) {
    // Test did not pass, make sure that test is for a file.
    $test = pift_test_get_summary($result['test_id']);
    if ($test['type'] == PIFT_TYPE_FILE) {
      $issue = node_load($test['nid']);
      if (pift_test_check_criteria_issue($issue)) {
        $test['cid'] = pift_test_get_comment_id($test);
        $file = file_load($test['id']);
        pift_cron_auto_followup($test['nid'], $test['cid'], $file->filename);
      }
    }
  }
}

/**
 * Create an auto followup comment on the specified node.
 *
 * @param integer $nid
 *   The node ID on which to post the followup on.
 * @param integer $cid
 *   The nodechanges comment ID where the failed patch was added.
 * @param string $filename
 *   The name of the patch file.
 */
function pift_cron_auto_followup($nid, $cid, $filename) {
  $node = node_load($nid);
  $node->field_issue_status[LANGUAGE_NONE][0]['value'] = PIFT_FOLLOWUP_FAIL;
  $node->nodechanges_uid = variable_get('project_issue_followup_user', drupal_anonymous_user()->uid);
  // TODO:  Deduce appropriate comment format (instead of hardcoded '1')
  $node->nodechanges_comment_body = array(
    'value' => theme('pift_auto_followup', array('type' => 'fail', 'nid' => $nid, 'cid' => $cid, 'filename' => $filename)),
    'format' => 1,
  );
  node_save($node);
}

/**
 * Convert an integer response code to a human-readable status value.
 *
 * @param integer $response_code
 *   The integer response code.
 *
 * @return string
 *   The Human readable representation of the provided reponse code.
 */
function pift_cron_response_code($response_code) {
  switch ($response_code) {
    case PIFR_RESPONSE_ACCEPTED:
      return t('Accepted');
    case PIFR_RESPONSE_INVALID_SERVER:
      return t('Invalid server (client key may have been disabled)');
    case PIFR_RESPONSE_DENIED:
      return t('Denied - ensure that test master is enabled and/or has reporting on');
    default:
      return 'Unknown';
  }
}
