<?php

/**
 * @file
 * Provides nodechanges integration for the PIFT module.
 */

/**
 * Implements hook_nodechanges_file_changes_element_alter().
 *
 * Adds test status and results to the extended_file_field formatter output on
 * nodechanges file changes fields.
 */
function pift_nodechanges_file_changes_element_alter(&$element, $context) {
  $retest = user_access('pift re-test files');
  $fids = array_keys($element['#files']);
  // If there are no files, bail.
  if (empty($fids)) {
    return;
  }
  // Retreive the test ids corresponding to these files.
  $test_ids = pift_test_get_test_ids($fids);
  // If there are no test results for these files, bail.
  if (empty($test_ids)) {
    return;
  }
  // Retrieve the test results corresponding to these tests.
  $tests = pift_test_get_summary_multiple($test_ids, 'file');
  $new_rows = array();
  // Loop through each file row, adding the test results as necessary.
  foreach ($element['#rows'] as $fid => $row) {
    if (!empty($test_ids[$fid]) && !in_array($element['#files'][$fid]->file_status, array('hidden', 'deleted'))) {
      $file = (array) $element['#files'][$fid];
      $test_id = $test_ids[$fid];
      $file['pift_id'] = $test_id;
      $file['pift_status'] = $tests[$test_id]['status'];
      $file['pift_message'] = $tests[$test_id]['message'];
      $file['pift_operations'] = array('view' => $test_id);
      $classes = array('', 'pift-queue', 'pift-retest', 'pift-fail', 'pift-pass');
      if ($retest && in_array($tests[$test_id]['status'], array(PIFT_STATUS_FAIL, PIFT_STATUS_PASS))) {
        $file['pift_operations']['retest'] = $test_id;
      }

      // Generate our replacement row items, maintaining any existing classes,
      // attributes, etc.
      if (!isset($row['data'])) {
        $new_rows[$fid] = array('data' => $row);
      }
      else {
        $new_rows[$fid] = $row;
      }

      // Add a rowspan to the first cell.
      if (!is_array($new_rows[$fid]['data'][0])) {
        $new_rows[$fid]['data'][0] = array('data' => $new_rows[$fid]['data'][0]);
      }

      $new_rows["$fid-results"] = array(
        'data' => array(
          array(),
          array(
            'data' => pift_extended_file_field_message_formatter($file, array()),
            'class' => array($classes[$file['pift_status']]),
          ),
          array(
            'data' => pift_extended_file_field_operations_formatter($file, array()),
          ),
        ),
      );
    }
    else {
      $new_rows[$fid] = $row;
    }
  }
  $element['#rows'] = $new_rows;
}
