<?php

/**
 * @file
 * Provides project related functions for the PIFT module.
 */

/**
 * Check if the project has testing enabled.
 *
 * @param integer $pid
 *   The project node ID.
 *
 * @return boolean
 *   Boolean indicating whether project has testing enabled.
 */
function pift_project_enabled($pid) {
  static $statuses = array();

  if (!isset($statuses[$pid])) {
    $nid = db_query('SELECT pid FROM {pift_project} WHERE pid = :pid',
        array(':pid' => $pid)
      )
      ->fetchField();
    $statuses[$pid] = ($nid !== FALSE);
  }
  return $statuses[$pid];
}

/**
 * Enable testing for a project.
 *
 * @param integer $pid
 *   The project node ID.
 */
function pift_project_enable_testing($pid) {
  db_query('INSERT INTO {pift_project} VALUES (:nid)', array(':nid' => $pid));
}

/**
 * Disable testing for a project.
 *
 * @param integer $pid
 *   The project node ID.
 */
function pift_project_disable_testing($pid) {
  db_delete('pift_project')
    ->condition('pid', $pid)
    ->execute();
}
