<?php

/**
 * @file
 * Provides versioncontrol integration for the PIFT module.
 */

/**
 * Implements hook_versioncontrol_code_arrival().
 *
 * Each time a new commit is made to a project with testing enabled, we need to
 * trigger a new branch test on the PIFR server.
 */
function pift_versioncontrol_code_arrival(VersioncontrolRepository $repository, VersioncontrolEvent $event) {
  // Ignore events for disabled projects and non-Git repos.
  if (!$repository instanceof VersioncontrolGitRepository || !pift_project_enabled($repository->project_nid)) {
    return;
  }

  module_load_include('cron.inc', 'pift');
  $api_versions = pift_core_api_versions();

  $branch_names = array();
  foreach ($event as $ref) {
    if (VERSIONCONTROL_GIT_REFTYPE_BRANCH === $ref->reftype) {
      $branch_names[] = $ref->refname;
    }
  }

  $rids = pift_cron_get_release($repository->project_nid, $branch_names);
  foreach ($rids as $rid) {
    // Ensure that one of the compatibility terms is present on the release node.
    $release = node_load($rid);
    $found = FALSE;
    // TODO:  Verify name of release core field.  Change to 'core compatibility' field?
    if (array_key_exists($release->taxonomy_vocabulary_6[$release['language']][0]['tid'], $api_versions)) {
      // Compatible term found, continue processing.
      $test_id = db_query('SELECT test_id
                                  FROM {pift_data}
                                  WHERE type = :type
                                  AND nid = :rid', array(':type' => PIFT_TYPE_BRANCH, ':rid' => $rid))->fetchField();

      // If existing test for release, queue it, otherwise add a new test.
      if ($test_id) {
        pift_test_requeue($test_id);
      }
      else if ($test_id !== 0) {
        $id = versioncontrol_release_get_release_label($release_nid);
        pift_test_add(PIFT_TYPE_BRANCH, $id, $rid);
      }
    }
  }
}
